package main

import (
	"context"
	"errors"
	"fmt"
	"github.com/dgraph-io/badger/v4"
	"github.com/disgoorg/disgo"
	"github.com/disgoorg/disgo/discord"
	"github.com/disgoorg/disgo/rest"
	"github.com/disgoorg/disgo/webhook"
	"github.com/disgoorg/log"
	"github.com/disgoorg/snowflake/v2"
	"grow.rievo.dev/discordBots/cmd/dealsbot/api"
	"grow.rievo.dev/discordBots/cmd/dealsbot/repository"
	"os"
	"os/signal"
	"reflect"
	"syscall"
	"time"
)

var (
	webhookID    = snowflake.GetEnv("webhook_id")
	webhookToken = os.Getenv("webhook_token")
)

func main() {
	// query different sources store to db
	// try to incorporate operagx apiUrl:
	// - https://gx-proxy.operacdn.com/content/free-games?_limit=300&_sort=order%3AASC
	// send messages to discord
	// ideas:
	// - https://github.com/TheLovinator1/discord-free-game-notifier
	// - https://gg.deals/news/free-gog-games/
	// - origin
	// - check ubisoft works

	log.SetLevel(log.LevelInfo)
	log.Info("starting dealsbot...")
	log.Info("disgo version: ", disgo.Version)

	client := webhook.New(webhookID, webhookToken)
	defer client.Close(context.TODO())

	repo := repository.InitDb()
	defer repo.Close()

	ticker := time.NewTicker(10 * time.Minute)
	tickerGC := time.NewTicker(15 * time.Minute)
	quit := make(chan struct{})

	go func() {
		for {
			select {
			case <-ticker.C:
				var apis []api.Api
				apis = append(apis, api.NewUbsioftApi(), api.NewEpicApi(), api.NewSteamApi(), api.NewGogApi(), api.NewHumbleBundleApi())
				for _, api := range apis {
					err := api.Load()
					if err != nil {
						log.Error(err)
					}
				}
				var deals []api.Deal
				for _, api := range apis {
					apiDeals := api.Get()
					deals = append(deals, apiDeals...)
				}

				for _, deal := range deals {
					retrievedDeal, _ := repo.GetValue(deal.Id)

					if deal.Id == retrievedDeal.Id {
						log.Debugf("%v is already published", deal.Id)
					} else if reflect.DeepEqual(deal, retrievedDeal) {
						log.Errorf("%v is published but not equal", deal.Id)
					} else {
						log.Infof("%v is new and will be published", deal.Id)
						go sendWebhook(client, deal)
						err := repo.SetValue(deal)
						if err != nil {
							log.Error(err)
						}
					}
				}

			case <-tickerGC.C:
				err := repo.RunGC()
				if err != nil && !errors.Is(err, badger.ErrNoRewrite) {
					log.Errorf("error with GC: %v", err)
				} else {
					log.Debug("GC successful")
				}
			case <-quit:
				ticker.Stop()
				tickerGC.Stop()
				return
			}
		}
	}()

	log.Infof("dealsbot is now running. Press CTRL-C to exit.")
	s := make(chan os.Signal, 1)
	signal.Notify(s, syscall.SIGINT, syscall.SIGTERM, os.Interrupt)
	<-s
}

func sendWebhook(client webhook.Client, deal api.Deal) {
	var status string

	status = fmt.Sprintf("currently free: %v\n", deal.Url)

	if _, err := client.CreateMessage(discord.NewWebhookMessageCreateBuilder().
		SetContent(status).Build(),
		rest.WithDelay(2*time.Second),
	); err != nil {
		log.Errorf("error sending message %v", err.Error())
	}
}
